import os
from distutils import ccompiler

from cffi import FFI


_ROOT = os.path.abspath(os.path.dirname(__file__))
_GUETZLI_CPP = os.path.join(_ROOT, "guetzli.cpp")
_GUETZLI_H = os.path.join(_ROOT, "guetzli.h")
_GUETZLI_STATIC_LIB = ccompiler.new_compiler() \
    .library_filename("guetzli", output_dir="guetzli")

ffibuilder = FFI()
ffibuilder.set_source(
        "pyguetzli._guetzli",
        open(_GUETZLI_CPP, "r").read(),
        extra_objects=[_GUETZLI_STATIC_LIB],
        include_dirs=[_ROOT, os.path.join(_ROOT, "..", "guetzli")],
        source_extension=".cpp"
        )
ffibuilder.cdef(open(_GUETZLI_H, "r").read())


if __name__ == "__main__":
    ffibuilder.compile(verbose=True)



#!/usr/bin/env python
# encoding: UTF-8

import io
import os
from distutils import ccompiler

from setuptools import setup, find_packages
from setuptools.command.build_py import build_py


class CustomBuildPy(build_py):

    def run(self):
        extra_cc_args = []

        if ccompiler.get_default_compiler() == "unix":
            os.environ["CPPFLAGS"] = "--std=c++11"
            extra_cc_args = ["-fPIC", "--std=c++11", "-O3"]

        if ccompiler.get_default_compiler() == "msvc":
            extra_cc_args = ["/O2"]

        compiler = ccompiler.new_compiler()
        compiler.set_include_dirs([
            "guetzli/",
            "guetzli/third_party/butteraugli/",
            ])
        objects = compiler.compile([
            "guetzli/guetzli/butteraugli_comparator.cc",
            "guetzli/guetzli/dct_double.cc",
            "guetzli/guetzli/debug_print.cc",
            "guetzli/guetzli/entropy_encode.cc",
            "guetzli/guetzli/fdct.cc",
            "guetzli/guetzli/gamma_correct.cc",
            "guetzli/guetzli/idct.cc",
            "guetzli/guetzli/jpeg_data.cc",
            "guetzli/guetzli/jpeg_data_decoder.cc",
            "guetzli/guetzli/jpeg_data_encoder.cc",
            "guetzli/guetzli/jpeg_data_reader.cc",
            "guetzli/guetzli/jpeg_data_writer.cc",
            "guetzli/guetzli/jpeg_huffman_decode.cc",
            "guetzli/guetzli/output_image.cc",
            "guetzli/guetzli/preprocess_downsample.cc",
            "guetzli/guetzli/processor.cc",
            "guetzli/guetzli/quality.cc",
            "guetzli/guetzli/quantize.cc",
            "guetzli/guetzli/score.cc",
            "guetzli/third_party/butteraugli/butteraugli/butteraugli.cc",
            ],
            extra_preargs=extra_cc_args)
        compiler.create_static_lib(objects, "guetzli", output_dir="guetzli")

        build_py.run(self)


long_description = ""
for readme_filename in ("README.rst", "README.md"):
    if os.path.isfile(readme_filename):
        with io.open(readme_filename, "r", encoding="utf-8") as f:
            long_description = f.read()

        break

setup(
    name="pyguetzli",
    version="1.0.8",
    description="Python bindings for Google's Guetzli, a JPEG encoder that optimises JPEG compression",  # noqa
    url="https://github.com/wanadev/pyguetzli",
    license="Apache-2.0",

    long_description=long_description,
    keywords="image jpeg optimize guetzli",

    author="Wanadev",
    author_email="contact@wanadev.fr",
    maintainer="Fabien LOISON, Alexis BREUST",

    packages=find_packages(),

    setup_requires=["cffi>=1.0.0"],
    install_requires=["cffi>=1.0.0"],
    extras_require={
        "PIL": ["pillow"],
        "dev": [
            "flake8",
            "Sphinx",
            "sphinx-rtd-theme",
            "pytest",
        ],
    },

    cffi_modules=["pyguetzli/guetzli_build.py:ffibuilder"],

    cmdclass={
        "build_py": CustomBuildPy,
    },
)



import pytest
from PIL import Image

from pyguetzli import pil_image


JPEG_MAGIC = b"\xFF\xD8\xFF\xE0\x00\x10JFIF"


class Test_process_pil_image(object):

    @pytest.mark.parametrize("image_path", [
            "./test/image.jpg",
            "./test/rgb.png",
            "./test/rgba.png",
            "./test/grayscale.png",
            "./test/indexed.png",
        ])
    def test_pil_image(self, image_path):
        image = Image.open(image_path)
        optimized_jpeg_bytes = pil_image.process_pil_image(image)
        assert isinstance(optimized_jpeg_bytes, bytes)
        assert optimized_jpeg_bytes.startswith(JPEG_MAGIC)

    def test_quality_param(self):
        image = Image.open("./test/image.jpg")

        optimized_jpeg_bytes_q100 = pil_image.process_pil_image(
                image, 100)
        optimized_jpeg_bytes_q50 = pil_image.process_pil_image(
                image, 50)
        assert len(optimized_jpeg_bytes_q50) < len(optimized_jpeg_bytes_q100)
